/**
 * impressEx.js Slide Engine
 * 
 * impressEx.js is an HTML5 slide engine, based on 'impress.js' project.
 * 
 * 
 * impress.js:
 * 		Copyright 2011-2012 Bartek Szopka (@bartaz)
 * 		Released under the MIT and GPL Licenses.
 * 		
 * 		author:  Bartek Szopka
 * 		
 * 		url:     http://bartaz.github.com/impress.js/
 * 		source:  http://github.com/bartaz/impress.js/
 * 
 * impressEx.js:
 * 		Copyright 2012 vIstaswx.com
 * 		License: MIT or GPL v2
 * 		
 * 		author:  @vistaswx
 * 		
 * 		source:  https://github.com/vistaswx/impressEx.js/
 */


(function(window, undefined)
{

	var pfx = (function()
	{
		var style = document.createElement('dummy').style, 
		prefixes = 'Webkit Moz O ms Khtml'.split(' '), 
		memory = {};

		return function(prop)
		{
			if (typeof memory[prop] === "undefined")
			{
				var ucProp = prop.charAt(0).toUpperCase() + prop.substr(1), 
				props = (prop + ' ' + prefixes.join(ucProp + ' ') + ucProp).split(' ');

				memory[prop] = null;
				
				for (var i in props)
				{
					if (style[props[i]] !== undefined)
					{
						memory[prop] = props[i];
						break;
					}
				}

			}

			return memory[prop];
		}
	})();

	var arrayify = function(a)
	{
		return [].slice.call(a);
	};
	
	var css = function(el, props)
	{
		var key, pkey;
		
		for (key in props)
		{
			if(props.hasOwnProperty(key))
			{
				pkey = pfx(key);
				
				if(pkey != null)
					el.style[pkey] = props[key];
			}
		}
		return el;
	}
	
	var $ = function(selector, context)
	{
		context = context || document;
		return context.querySelector(selector);
	};
	
	var $$ = function(selector, context)
	{
		context = context || document;
		return arrayify(context.querySelectorAll(selector));
	};
	
	var translate = function(t)
	{
		return " translate3d(" + t.x + "px," + t.y + "px," + t.z + "px) ";
	};
	
	var rotate = function(r, revert)
	{
		var rX = " rotateX(" + r.x + "deg) ", rY = " rotateY(" + r.y + "deg) ", rZ = " rotateZ(" + r.z + "deg) ";

		return revert ? rZ + rY + rX : rX + rY + rZ;
	};
	
	var scale = function(s)
	{
		return " scale(" + s + ") ";
	}
	
	var ua = navigator.userAgent.toLowerCase();
	var impressSupported = (pfx("perspective") != null ) && (ua.search(/(iphone)|(ipod)|(ipad)|(android)/) == -1 );

	// DOM ELEMENTS

	var impress = byId("impressEx");

	if(!impressSupported)
	{
		impress.className = "impress-not-supported";
		return;
	}
	else
	{
		impress.className = "";
	}

	var canvas = document.createElement("div");
	canvas.className = "canvas";
	
	arrayify(impress.childNodes).forEach(function(el)
	{
		canvas.appendChild(el);
	});
	impress.appendChild(canvas);

	var steps = $$(".slide_step", impress);

	// SETUP
	// set initial values and defaults

	document.documentElement.style.height = "100%";

	css(document.body,
	{
		height : "100%",
		overflow : "hidden"
	});

	var props =
	{
		position : "absolute",
		transformOrigin : "top left",
		transition : "all .5s ease-in-out",
		transformStyle : "preserve-3d"
	}

	css(impress, props);
	css(impress,
	{
		top : "50%",
		left : "50%",
		perspective : "1000px"
	});
	css(canvas, props);

	var current =
	{
		translate :
		{
			x : 0,
			y : 0,
			z : 0
		},
		rotate :
		{
			x : 0,
			y : 0,
			z : 0
		},
		scale : 1
	};

	steps.forEach(function(el, idx)
	{
		var data = el.dataset, step =
		{
			translate :
			{
				x : data.x || 0,
				y : data.y || 0,
				z : data.z || 0
			},
			rotate :
			{
				x : data.rotateX || 0,
				y : data.rotateY || 0,
				z : data.rotateZ || data.rotate || 0
			},
			scale : data.scale || 1
		};

		el.stepData = step;

		if(!el.id)
		{
			el.id = "step-" + (idx + 1);
		}

		css(el,
		{
			position : "absolute",
			transform : "translate(-50%,-50%)" + translate(step.translate) + rotate(step.rotate) + scale(step.scale),
			transformStyle : "preserve-3d"
		});

	});
	
	var ImpressEx = window.ImpressEx = {};
	var callbacks = {};
	
	var active = null;
	var hashTimeout = null;

	var lastel;
	
	function addCallback(namespace, callback)
	{
		if(!( typeof callback == 'function'))
			return;

		if(callbacks[namespace] == undefined)
			callbacks[namespace] = [];
		callbacks[namespace].push(callback);
	}

	function triggerCallback(namespace, params, tPoint)
	{
		var k = callbacks[namespace];
		if(k == undefined)
			return;

		for(var i = 0; i < k.length; k++)
		{
			if( typeof k[i] == 'function')
			{
				if(k[i].apply(tPoint, params) === false)
					return false;
			}
		}
	}
	
	ImpressEx.prototype = 
	{
		//Before switching slide
		before: function(slideIdName, callback)
		{
			if(slideIdName == undefined)
				slideIdName = '__global__';
	
			addCallback('before_' + slideIdName.toString(), callback);
		},
		
		//After switched slide
		after: function(slideIdName, callback)
		{
			if(slideIdName == undefined)
				slideIdName = '__global__';
	
			addCallback('before_' + slideIdName.toString(), callback);
		},
		
		//After special key pressed
		key: function(slideIdName, callback)
		{
			if(slideIdName == undefined)
				slideIdName = '__global__';
	
			addCallback('key_' + slideIdName.toString(), callback);
		},
		
		//Before leaving the slide
		leave: function(slideIdName, callback)
		{
			if(slideIdName == undefined)
				slideIdName = '__global__';
	
			addCallback('leave_' + slideIdName.toString(), callback);
		},
		
		//Start the slide
		start: function()
		{
			select(getElementFromUrl() || steps[0]);
		},
		
		//Switch to next slide
		next: function()
		{
			var next = active;
			
			next = steps.indexOf(active) + 1;
			next = next < steps.length ? steps[next] : steps[0];
			
			select(next);
		},
		
		//Switch to previous slide
		prev: function()
		{
			var prev = active;
			
			prev = steps.indexOf(active) - 1;
			prev = prev >= 0 ? steps[prev] : steps[steps.length - 1];
			
			select(prev);
		}
	};
	
	var select = function(el)
	{
		if(!el || !el.stepData || el == active)
		{
			// selected element is not defined as step or is already active
			return false;
		}

		if(lastel != null)
		{
			if(triggerCallback('leave_' + lastel.id, [lastel.id], lastel) === false)
				return false;
			if(triggerCallback('leave___global__', [lastel.id], lastel) === false)
				return false;
		}
		
		lastel = el;

		// Sometimes it's possible to trigger focus on first link with some keyboard
		// action.
		// Browser in such a case tries to scroll the page to make this element visible
		// (even that body overflow is set to hidden) and it breaks our careful
		// positioning.
		//
		// So, as a lousy (and lazy) workaround we will make the page scroll back to the
		// top
		// whenever slide is selected
		//
		// If you are reading this and know any better way to handle it, I'll be glad to
		// hear about it!
		window.scrollTo(0, 0);

		if(triggerCallback('before_' + el.id, [el.id, el.dataset['d_count']], el) === false)
			return false;
		if(triggerCallback('before___global__', [el.id, el.dataset['d_count']], el) === false)
			return false;

		el.dataset['d_count'] = 0;

		var step = el.stepData;

		if(active)
		{
			active.classList.remove("active");
		}
		el.classList.add("active");

		impress.className = "step-" + el.id;

		ImpressEx.activeSlideId = el.id;

		// `#/step-id` is used instead of `#step-id` to prevent default browser
		// scrolling to element in hash
		//
		// and it has to be set after animation finishes, because in chrome it
		// causes transtion being laggy
		//window.clearTimeout( hashTimeout );
		//hashTimeout = window.setTimeout(function () {
		window.location.hash = "#/" + el.id;
		//}, 1000);

		var target =
		{
			rotate :
			{
				x : -parseInt(step.rotate.x, 10),
				y : -parseInt(step.rotate.y, 10),
				z : -parseInt(step.rotate.z, 10)
			},
			translate :
			{
				x : -step.translate.x,
				y : -step.translate.y,
				z : -step.translate.z
			},
			scale : 1 / parseFloat(step.scale)
		};

		var zoomin = target.scale >= current.scale;

		css(impress,
		{
			// to keep the perspective look similar for different scales
			// we need to 'scale' the perspective, too
			perspective : step.scale * 1000 + "px",
			transform : scale(target.scale),
			transitionDelay : ( zoomin ? "500ms" : "0ms")
		});

		css(canvas,
		{
			transform : rotate(target.rotate, true) + translate(target.translate),
			transitionDelay : ( zoomin ? "0ms" : "500ms")
		});
		current = target;
		active = el;

		triggerCallback('after_' + el.id, [el.id]);
		triggerCallback('after___global__', [el.id]);

		return el;
	}
	// EVENTS

	document.addEventListener("keydown", function(event)
	{
		var next = active;

		switch( event.which )
		{
			case 33:// pg up
			case 37:// left
			case 38:// up
				ImpressEx.prev();
				event.preventDefault();

				break;
			case 34:// pg down
			case 39:// right
			case 40:// down
				ImpressEx.next();
				event.preventDefault();

				break;
			case 32:// space
				triggerCallback('key_' + active.id, [active.id, parseInt(active.dataset['d_count'])], active);
				triggerCallback('key___global__', [active.id, parseInt(active.dataset['d_count'])], active);
				
				break;
		}
	}, false);

	document.addEventListener("click", function(event)
	{
		// event delegation with "bubbling"
		// check if event target (or any of its parents is a link or a step)
		var target = event.target;
		while((target.tagName != "A") && (!target.stepData) && (target != document.body))
		{
			target = target.parentNode;
		}

		if(target.tagName == "A")
		{
			var href = target.getAttribute("href");

			// if it's a link to presentation step, target this step
			if(href && href[0] == '#')
			{
				target = byId(href.slice(1));
			}
		}

		if(select(target))
		{
			event.preventDefault();
		}
	}, false);
	
	var getElementFromUrl = function()
	{
		// get id from url # by removing `#` or `#/` from the beginning,
		// so both "fallback" `#slide-id` and "enhanced" `#/slide-id` will work
		return byId(window.location.hash.replace(/^#\/?/, ""));
	}
	
	window.addEventListener("hashchange", function()
	{
		select(getElementFromUrl());
	}, false);


})(window);